@@ -33,13 +33,13 @@ import pkg_resources
 osprofiler_profiler = importutils.try_import("osprofiler.profiler")
 osprofiler_web = importutils.try_import("osprofiler.web")
 
+import novaclient
 from novaclient import api_versions
 from novaclient import exceptions
 from novaclient import extension as ext
 from novaclient.i18n import _
 from novaclient import utils
 
-
 # TODO(jichenjc): when an extension in contrib is moved to core extension,
 # Add the name into the following list, then after last patch merged,
 # remove the whole function
@@ -48,6 +48,9 @@ extensions_ignored_name = ["__init__"]
 
 class SessionClient(adapter.LegacyJsonAdapter):
 
+    client_name = 'python-novaclient'
+    client_version = novaclient.__version__
+
     def __init__(self, *args, **kwargs):
         self.times = []
         self.timings = kwargs.pop('timings', False)
