@@ -34,6 +34,7 @@ class ServersTest(utils.FixturedTestCase):
     client_fixture_class = client.V1
     data_fixture_class = data.V1
     api_version = None
+    supports_files = True
 
     def setUp(self):
         super(ServersTest, self).setUp()
@@ -46,39 +47,44 @@ class ServersTest(utils.FixturedTestCase):
         """
         return None
 
-    def test_list_servers(self):
+    def test_list_all_servers(self):
         sl = self.cs.servers.list()
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('GET', '/servers/detail')
+        self.assert_called('GET', '/servers/detail', pos=-2)
+        self.assert_called('GET', '/servers/detail?marker=9012')
         for s in sl:
             self.assertIsInstance(s, servers.Server)
 
     def test_filter_servers_unicode(self):
         sl = self.cs.servers.list(search_opts={'name': u't€sting'})
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('GET', '/servers/detail?name=t%E2%82%ACsting')
-        for s in sl:
-            self.assertIsInstance(s, servers.Server)
-
-    def test_list_all_servers(self):
-        # use marker just to identify this call in fixtures
-        sl = self.cs.servers.list(limit=-1, marker=1234)
-        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-
-        self.assertEqual(2, len(sl))
-
-        self.assertEqual(self.requests_mock.request_history[-2].method, 'GET')
-        self.assertEqual(self.requests_mock.request_history[-2].path_url,
-                         '/servers/detail?marker=1234')
-        self.assert_called('GET', '/servers/detail?marker=5678')
-
+        self.assert_called(
+            'GET',
+            '/servers/detail?name=t%E2%82%ACsting',
+            pos=-2)
+        self.assert_called(
+            'GET',
+            '/servers/detail?marker=9012&name=t%E2%82%ACsting')
         for s in sl:
             self.assertIsInstance(s, servers.Server)
 
     def test_list_servers_undetailed(self):
         sl = self.cs.servers.list(detailed=False)
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
-        self.assert_called('GET', '/servers')
+        self.assert_called('GET', '/servers', pos=-2)
+        self.assert_called('GET', '/servers?marker=5678')
+        for s in sl:
+            self.assertIsInstance(s, servers.Server)
+
+    def test_list_servers_with_marker(self):
+        sl = self.cs.servers.list(marker=1234)
+        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
+
+        self.assertEqual(2, len(sl))
+
+        self.assert_called('GET', '/servers/detail?marker=1234', pos=-2)
+        self.assert_called('GET', '/servers/detail?marker=9012')
+
         for s in sl:
             self.assertIsInstance(s, servers.Server)
 
@@ -88,6 +94,17 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_called('GET', '/servers/detail?limit=2&marker=1234')
         for s in sl:
             self.assertIsInstance(s, servers.Server)
+        self.assertEqual(2, len(sl))
+
+    def test_list_servers_with_limit_above_max_limit(self):
+        # use limit=3 to trigger paging simulation on backend fixture side
+        sl = self.cs.servers.list(limit=3)
+        self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('GET', '/servers/detail?limit=3', pos=-2)
+        self.assert_called('GET', '/servers/detail?limit=1&marker=5678')
+        for s in sl:
+            self.assertIsInstance(s, servers.Server)
+        self.assertEqual(3, len(sl))
 
     def test_list_servers_sort_single(self):
         sl = self.cs.servers.list(sort_keys=['display_name'],
@@ -95,7 +112,10 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_request_id(sl, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called(
             'GET',
-            '/servers/detail?sort_dir=asc&sort_key=display_name')
+            '/servers/detail?sort_dir=asc&sort_key=display_name', pos=-2)
+        self.assert_called(
+            'GET',
+            '/servers/detail?marker=9012&sort_dir=asc&sort_key=display_name')
         for s in sl:
             self.assertIsInstance(s, servers.Server)
 
@@ -106,6 +126,11 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_called(
             'GET',
             ('/servers/detail?sort_dir=asc&sort_dir=desc&'
+             'sort_key=display_name&sort_key=id'),
+            pos=-2)
+        self.assert_called(
+            'GET',
+            ('/servers/detail?marker=9012&sort_dir=asc&sort_dir=desc&'
              'sort_key=display_name&sort_key=id'))
         for s in sl:
             self.assertIsInstance(s, servers.Server)
@@ -126,6 +151,12 @@ class ServersTest(utils.FixturedTestCase):
         self.assertEqual(s1._info, s2._info)
 
     def test_create_server(self):
+        kwargs = {}
+        if self.supports_files:
+            kwargs['files'] = {
+                '/etc/passwd': 'some data',             # a file
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
+            }
         s = self.cs.servers.create(
             name="My server",
             image=1,
@@ -133,11 +164,8 @@ class ServersTest(utils.FixturedTestCase):
             meta={'foo': 'bar'},
             userdata="hello moto",
             key_name="fakekey",
-            files={
-                '/etc/passwd': 'some data',                 # a file
-                '/tmp/foo.txt': six.StringIO('data'),   # a stream
-            },
-            nics=self._get_server_create_default_nics()
+            nics=self._get_server_create_default_nics(),
+            **kwargs
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
@@ -253,23 +281,32 @@ class ServersTest(utils.FixturedTestCase):
             self.assertIsInstance(s, servers.Server)
 
     def test_create_server_userdata_file_object(self):
+        kwargs = {}
+        if self.supports_files:
+            kwargs['files'] = {
+                '/etc/passwd': 'some data',             # a file
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
+            }
         s = self.cs.servers.create(
             name="My server",
             image=1,
             flavor=1,
             meta={'foo': 'bar'},
             userdata=six.StringIO('hello moto'),
-            files={
-                '/etc/passwd': 'some data',                 # a file
-                '/tmp/foo.txt': six.StringIO('data'),   # a stream
-            },
             nics=self._get_server_create_default_nics(),
+            **kwargs
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
         self.assertIsInstance(s, servers.Server)
 
     def test_create_server_userdata_unicode(self):
+        kwargs = {}
+        if self.supports_files:
+            kwargs['files'] = {
+                '/etc/passwd': 'some data',             # a file
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
+            }
         s = self.cs.servers.create(
             name="My server",
             image=1,
@@ -277,17 +314,20 @@ class ServersTest(utils.FixturedTestCase):
             meta={'foo': 'bar'},
             userdata=six.u('こんにちは'),
             key_name="fakekey",
-            files={
-                '/etc/passwd': 'some data',                 # a file
-                '/tmp/foo.txt': six.StringIO('data'),   # a stream
-            },
             nics=self._get_server_create_default_nics(),
+            **kwargs
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
         self.assertIsInstance(s, servers.Server)
 
     def test_create_server_userdata_utf8(self):
+        kwargs = {}
+        if self.supports_files:
+            kwargs['files'] = {
+                '/etc/passwd': 'some data',             # a file
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
+            }
         s = self.cs.servers.create(
             name="My server",
             image=1,
@@ -295,11 +335,8 @@ class ServersTest(utils.FixturedTestCase):
             meta={'foo': 'bar'},
             userdata='こんにちは',
             key_name="fakekey",
-            files={
-                '/etc/passwd': 'some data',                 # a file
-                '/tmp/foo.txt': six.StringIO('data'),   # a stream
-            },
             nics=self._get_server_create_default_nics(),
+            **kwargs
         )
         self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers')
@@ -323,6 +360,12 @@ class ServersTest(utils.FixturedTestCase):
         self.assertEqual(test_password, body['server']['adminPass'])
 
     def test_create_server_userdata_bin(self):
+        kwargs = {}
+        if self.supports_files:
+            kwargs['files'] = {
+                '/etc/passwd': 'some data',             # a file
+                '/tmp/foo.txt': six.StringIO('data'),   # a stream
+            }
         with tempfile.TemporaryFile(mode='wb+') as bin_file:
             original_data = os.urandom(1024)
             bin_file.write(original_data)
@@ -335,11 +378,8 @@ class ServersTest(utils.FixturedTestCase):
                 meta={'foo': 'bar'},
                 userdata=bin_file,
                 key_name="fakekey",
-                files={
-                    '/etc/passwd': 'some data',                 # a file
-                    '/tmp/foo.txt': six.StringIO('data'),   # a stream
-                },
                 nics=self._get_server_create_default_nics(),
+                **kwargs
             )
             self.assert_request_id(s, fakes.FAKE_REQUEST_ID_LIST)
             self.assert_called('POST', '/servers')
@@ -1500,3 +1540,29 @@ class ServersV256Test(ServersV254Test):
         ex = self.assertRaises(TypeError,
                                s.migrate, host='target-host')
         self.assertIn('host', six.text_type(ex))
+
+
+class ServersV257Test(ServersV256Test):
+    """Tests the servers python API bindings with microversion 2.57 where
+    personality files are deprecated.
+    """
+    api_version = "2.57"
+    supports_files = False
+
+    def test_create_server_with_files_fails(self):
+        ex = self.assertRaises(
+            exceptions.UnsupportedAttribute, self.cs.servers.create,
+            name="My server", image=1, flavor=1,
+            files={
+                '/etc/passwd': 'some data',  # a file
+                '/tmp/foo.txt': six.StringIO('data'),  # a stream
+            }, nics='auto')
+        self.assertIn('files', six.text_type(ex))
+
+    def test_rebuild_server_name_meta_files(self):
+        files = {'/etc/passwd': 'some data'}
+        s = self.cs.servers.get(1234)
+        ex = self.assertRaises(
+            exceptions.UnsupportedAttribute, s.rebuild, image=1, name='new',
+            meta={'foo': 'bar'}, files=files)
+        self.assertIn('files', six.text_type(ex))
